package gui.view;

import javafx.beans.value.ChangeListener;
import javafx.beans.value.ObservableValue;
import javafx.fxml.FXML;
import javafx.scene.control.Slider;
import javafx.scene.control.ChoiceBox;
import javafx.scene.control.ToggleButton;
import javafx.scene.image.ImageView;
import gui.MainApp;
import gui.model.Engine;

/*
 * Die Klasse EngineController stellt den Controller fr die Bedienelemente zur Steuerung der Lokomotiven bereit. 
 */

public class EngineController {
	@FXML
	private ImageView engineImage;
	@FXML
	private Slider engineSpeedSlider;
	@FXML
	private ChoiceBox<Engine> engineChoiceBox;
	@FXML
	private ToggleButton engineRevButton, engineFwdButton;
	
	// Reference to the main application.
    private MainApp mainApp;
    private Engine eng;

    /**
     * The constructor.
     * The constructor is called before the initialize() method.
     */
    public EngineController() {
    }

    /**
     * Initializes the controller class. This method is automatically called
     * after the fxml file has been loaded.
     */
    @FXML
    private void initialize() {
    	engineSpeedSlider.setValue(0);
        engineFwdButton.setSelected(true); // initializes the forward Button as pressed - default direction
        
        // updates the local reference of the selected engine to the Engine selected in the ChoiceBox
        engineChoiceBox.getSelectionModel().selectedItemProperty().addListener(
        		(observable, oldValue, newValue) -> setSelectedEngine(newValue)); 
        
        engineSpeedSlider.valueProperty().addListener(new ChangeListener<Number>() {
        	public void changed(ObservableValue<? extends Number> ov,
                    Number old_val, Number new_val) {
                        eng.setSpeed(new_val.intValue()*10);
                        mainApp.setEngineSpeed(eng);
                    	updateEngineStatus();
        				}
        			});
    }

    /**
     * Is called by the main application to give a reference back to itself.
     * 
     * @param mainApp
     */
    public void setMainApp(MainApp mainApp) {
        this.mainApp = mainApp;
        engineChoiceBox.setItems(mainApp.getEngines());
        engineChoiceBox.setValue(mainApp.getEngines().get(0));
        engineImage.setImage(mainApp.getEngines().get(0).getImg().get());
    }
    
    // Is called if the state of the direction buttons changes and updates the engine model accordingly
    public void handleEngineDirection(){
    	eng.setDirection(engineFwdButton.isSelected());
    	mainApp.setEngineDirection(eng);
    	updateEngineStatus();
    }
    
    // updates the GUI to the values of the currently selected engine
    public void updateEngineStatus(){
    	engineImage.setImage(eng.getImg().get());
    	engineSpeedSlider.setValue(eng.getSpeed().get()/10);
    	
    	StringBuilder status = new StringBuilder("Letztes Kommando: ");
    	status.append(eng.toString() + " ");
    	status.append(eng.getSpeed().get()/10 + "% ");
    	engineFwdButton.setSelected(eng.getDirection().get());
    	engineRevButton.setSelected(!eng.getDirection().get());
    	if (eng.getDirection().get())
    		status.append("vorwrts");
    	else
    		status.append("rckwrts");
    	mainApp.setStatus(status.toString());
    }
    
    
    /**
     * updates the local reference of the selected engine to the given one
     * 
     * @param Engine
     */
    public void setSelectedEngine(Engine eng){
    	this.eng = eng;
    	mainApp.setStatus(eng.toString() + " ausgewhlt");
    	updateEngineStatus();
    }
    
    public Engine getSelectedEngine(){
    	return eng;
    }
    
}
