package server;

import java.util.ArrayList;

/*
 * Diese Klasse stellt das Modell fuer eine Weiche auf Serverseite dar.
 * Weiterhin lassen sich über eine lokale Instanz des Maerklin-Protokolls
 * Befehle an die Maerklin-C2-Steuerung senden.
 */
public class Switch {
	
	// Static array containing all created instances of the object
	static ArrayList<Switch> switches;
	
	private int switchID;
	int switchDirection;
	
	// Private locale instance of the maerklin-protocol to send commands to the control-station
	private MaerklinProtocol udpProtocol;
	
	// Constructor
	public Switch(){
		this(0, 1);
		
		// For offline-debugging
		/*if (switches == null) {
			switches = new ArrayList<Switch>();
		}
		udpProtocol = new MaerklinProtocol();
		listener = new UDPListener();
		switches.add(this);
		switchID = 0;
		switchDirection = 1;*/
	}
	
	// Constructor with some initial data
	public Switch(int id, int direction){
		if (switches == null) {
			switches = new ArrayList<Switch>();
		}
		udpProtocol = new MaerklinProtocol();
		switches.add(this);
		try {
			setSwitchID(id);
			setSwitchDirection(direction);
		}
		catch(Exception e){
			System.out.println(e.getMessage());
		}
	}
	
	// Sets the Maerklin-ID
	public void setSwitchID(int newSwitchID){
		switchID = newSwitchID;
	}
	
	// Sets the switch-direction and sends the respective command to the control-station
	public void setSwitchDirection(int newSwitchDirection) throws Exception {
		if (newSwitchDirection == 0x01 || newSwitchDirection == 0x00){
			switchDirection = newSwitchDirection;
		}
		else {
			Exception e = new Exception("No viable switch direction!");
			throw e;
		}
		udpProtocol.changeSwitchDirection(switchID, switchDirection);		
	}
	
	// Changes the switch-direction and calls setSwitchDirection with the new vdirection-alue
	public void changeSwitchDirection() throws Exception {
		if (switchDirection == 0x00){
			setSwitchDirection(0x01);
		}
		else {
			setSwitchDirection(0x00);
		}
	}
	
	// Returns the Maerklin-ID
	public int getSwitchID(){
		return switchID;
	}
	
	// Returns the switch-direction
	public int getSwitchDirection(){
		return switchDirection;
	}

}
