package gui.model;

import javafx.beans.property.IntegerProperty;
import javafx.beans.property.ObjectProperty;

import javafx.beans.property.BooleanProperty;
import javafx.beans.property.SimpleIntegerProperty;
import javafx.beans.property.SimpleObjectProperty;
import javafx.beans.property.SimpleBooleanProperty;
import javafx.beans.property.SimpleStringProperty;
import javafx.beans.property.StringProperty;
import javafx.scene.image.Image;

/*
 * Modell-Klasse fuer einen Zug.
 */
public class Engine {

    private final StringProperty name;
    private final IntegerProperty id;
    private final IntegerProperty speed;
    private final BooleanProperty fwd;
    private final ObjectProperty<Image> img;

    // Default constructor
    public Engine() {
        this(null, null, null);
    }
    
    public Engine(String name, Integer id) {
        this(name, id, null);
    }

    // Constructor with some initial data
    public Engine(String name, Integer id, String imgPath) {
        this.name = new SimpleStringProperty(name);
        this.id = new SimpleIntegerProperty(id);
        this.img = new SimpleObjectProperty<Image>(new Image("file:" + imgPath));
        
        // Initialize the switch in a straigt (=false) state
        this.fwd = new SimpleBooleanProperty(true);
        this.speed = new SimpleIntegerProperty(0);
      
	}
    
    // Overrides the toString-functionality to return the instances value of name
    @Override
    public String toString() {
      return name.getValue();
    }
    
    // Returns the name
    public StringProperty getName() {
		return name;
	}
    
    // Returns the Maerklin-ID
    public IntegerProperty getMaerklinID() {
		return id;
	}

    // Returns the direction
	public BooleanProperty getDirection() {
		return fwd;
	}
	
	// Sets the direction
	public void setDirection(Boolean fwd) {
		this.fwd.set(fwd);
	}
	
	// Returns the speed
	public IntegerProperty getSpeed() {
		return speed;
	}
	
	// Sets the speed
	public void setSpeed(Integer speed) {
		this.speed.set(speed);
	}
	
	// Returns the objects matching image
	public ObjectProperty<Image> getImg(){
		return img;
	}
	
	// Sets the objects matchin image
	public void setImg(Image img){
		this.img.set(img);
	}
}
