package gui.view;

import javafx.beans.value.ChangeListener;
import javafx.beans.value.ObservableValue;
import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;
import javafx.scene.control.Label;
import javafx.scene.image.Image;
import javafx.scene.image.ImageView;
import javafx.scene.layout.HBox;

import java.io.IOException;

import org.controlsfx.control.ToggleSwitch;
import gui.MainApp;
import gui.model.Switch;

/*
 * Die Klasse SwitchControl stellt ein Sonderbedienelement dar. Sie ldt die Bedienelemente fr jeweils eine Weiche
 * und fungiert gleichzeitig als deren Controller. Ihr wird im Konstruktor das zugehrige Weichen-Objekt bergeben. 
 */

public class SwitchControl extends HBox{
	@FXML
	private Label nameLabel;
	@FXML
	private ToggleSwitch stateToggleSwitch;
	@FXML
	private ImageView straightIcon;
	@FXML
	private ImageView bentIcon;
		
	// Reference to the main application.
    private MainApp mainApp;
    private Switch sw;
    private Image straightActive = new Image("file:res/MagIcon_00_01_a.png");
    private Image straightInactive = new Image("file:res/MagIcon_00_01_i.png");
    private Image bentActive = new Image("file:res/MagIcon_00_00_a.png");
    private Image bentInactive = new Image("file:res/MagIcon_00_00_i.png");
    

    /**
     * The constructor.
     * The constructor is called before the initialize() method.
     * Unique in that it does also load the Switch.fxml, which is usally done in an
     * higher class, not in the controller itself.
     */
    public SwitchControl(Switch sw) {
    	this.sw = sw;
    	  	
    	FXMLLoader fxmlLoader = new FXMLLoader(getClass().getResource("Switch.fxml"));
        fxmlLoader.setRoot(this);
        fxmlLoader.setController(this);
        sw.setController(this);

        try {
            fxmlLoader.load();
        } catch (IOException exception) {
            throw new RuntimeException(exception);
        }
    }

    /**
     * Initializes the controller class. This method is automatically called
     * after the fxml file has been loaded.
     */
    @FXML
    private void initialize() {
        nameLabel.setText(sw.toString());
        straightIcon.setImage(straightActive);
        bentIcon.setImage(bentInactive);
        stateToggleSwitch.selectedProperty().addListener(new ChangeListener<Boolean>(){
			public void changed(ObservableValue<? extends Boolean> observable, Boolean oldValue, Boolean newValue) {
				sw.setState(newValue);
				mainApp.setSwitchState(sw);
	        	updateSwitchStatus();
			}
        });
    }

    /**
     * Is called by the main application to give a reference back to itself.
     * 
     * @param mainApp
     */
    public void setMainApp(MainApp mainApp) {
        this.mainApp = mainApp;
    }
   
    // updates the GUI elements to reflect the current state of the switch
    public void updateSwitchStatus(){
    	stateToggleSwitch.setSelected(sw.getState().get());
    	
    	StringBuilder status = new StringBuilder("Letztes Kommando: ");
    	status.append(sw.toString() + " ");
    	if (!sw.getState().get()){
    		status.append("gerade");
    		straightIcon.setImage(straightActive);
    		bentIcon.setImage(bentInactive);
    	}
    	else{
    		status.append("krumm");
    		straightIcon.setImage(straightInactive);
    		bentIcon.setImage(bentActive);
    	}
    	mainApp.setStatus(status.toString());
    }
}
