package server;


import java.util.ArrayList;

/*
 * Diese Klasse stellt das Modell fuer einen Zug auf Serverseite dar.
 * Weiterhin lassen sich ueber eine lokale Instanz des Maerklin-Protokolls
 * Befehle an die Maerklin-C2-Steuerung senden.
 */
public class Engine {
	
	// Static array containing all created instances of the object
	static ArrayList<Engine> engines;
	
	private int engineID;
	int engineDirection;
	int engineSpeed;
	
	// Private locale instance of the maerklin-protocol to send commands to the control-station
	private MaerklinProtocol udpProtocol;
	
	// Constructor
	public Engine(){
		this(0, 1, 0);
		
		// For offline-debugging
		/*if (engines == null) {
			engines = new ArrayList<Engine>();
		}
		udpProtocol = new MaerklinProtocol();
		listener = new UDPListener();
		engines.add(this);
		engineID = 0;
		engineDirection = 1;
		engineSpeed = 0;*/
	}
	
	// Constructor with some initial data
	public Engine(int id, int direction, int speed){
		if (engines == null) {
			engines = new ArrayList<Engine>();
		}
		udpProtocol = new MaerklinProtocol();
		engines.add(this);
		try {
			setEngineID(id);
			setEngineDirection(direction);
			setEngineSpeed(speed);
		}
		catch(Exception e){
			System.out.println(e.getMessage());
		}
	}
	
	// Sets the Maerklin-ID
	public void setEngineID(int newID){
		engineID = newID;
	}
	
	// Sets the engine-direction and sends the respective command to the control-station; a 3 as the direction-parameter causes a switch of the direction
	public void setEngineDirection(int newEngineDirection) throws Exception {
		if (newEngineDirection == 1 || newEngineDirection == 2){
			engineDirection = newEngineDirection;
		}
		else if (newEngineDirection == 3){
			if (engineDirection == 1){
				engineDirection = 2;
			}
			else {
				engineDirection = 1;
			}
		}
		else {
			Exception e = new Exception("No viable direction!");
			throw e;
		}
		udpProtocol.changeEngineDirection(engineID, engineDirection);
	}
	
	
	// Sets the engine-speed and sends the respective command to the control-station
	public void setEngineSpeed(int newEngineSpeed) throws Exception {
		if (newEngineSpeed <= 1000 && newEngineSpeed >= 0){
			engineSpeed = newEngineSpeed;
		}
		else {
			Exception e = new Exception("No viable speed value!");
			throw e;
		}
		udpProtocol.changeEngineSpeed(engineID, engineSpeed);
	}
	
	// Increases the engine-speed by 50 and calls changeEngineSpeed with the new speed-value
	public void increaseEngineSpeed() throws Exception {
		engineSpeed = engineSpeed+50;
		udpProtocol.changeEngineSpeed(engineID, engineSpeed);
	}
	
	// Decreases the engine-speed by 50 and calls changeEngineSpeed with the new speed-value
	public void decreaseEngineSpeed() throws Exception{
		engineSpeed = engineSpeed-50;
		udpProtocol.changeEngineSpeed(engineID, engineSpeed);
	}
	
	// Returns the Maerklin-ID
	public int getEngineID(){
		return engineID;
	}
	
	// Returns the engine-direction
	public int getEngineDirection(){
		return engineDirection;
	}
	
	// Returns the engine-speed
	public int getEngineSpeed(){
		return engineSpeed;
	}

}
