package server;

import java.net.*;

/*
 * Diese Klasse implementiert das Maerklin-Protokoll zur Kommunikation
 * mit der Maerklin-CS2-Steuerung. Die Datagramme werden mittels UDP
 * gesendet; fuer jedes Datagram wird ein neuer Sockel geoeffnet.
 */
public class MaerklinProtocol {
	
	// Setting the input-port of the maerklin-control-station
	private final static int port = 15731;
	
	// IP of the maerklin-control-station
	private final static String ip ="192.168.74.118";
	
	// Constructor
	public MaerklinProtocol(){
		//Nothing to do...
	}
	
	// Tries to open a UDP-socket and send the datagram to the maerklin-control-station
	private void writeDatagram(String ip, int port, byte[] data){
		//System.out.printf("\nGesendet: " + DatatypeConverter.printHexBinary(data));
		try(DatagramSocket socket = new DatagramSocket()){
			socket.setReuseAddress(true);
			InetAddress  address = InetAddress.getByName(ip);
			DatagramPacket outgoingData = new DatagramPacket(data, data.length, address, port);
			socket.send(outgoingData);
		} 
		catch (Exception e) {
			e.printStackTrace();
		} 
	}
	
	// Resets the emergency-stop (re-enables power)
	public void systemGo(){
		byte[] out = {(byte) 0x00, (byte) 0x00, (byte) 0x07, (byte) 0x78, (byte) 0x05, (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x01, (byte) 0x00, (byte) 0x00, (byte) 0x00};
		this.writeDatagram(ip, port, out);
	}
	
	// Emergency-stop (cuts the power)	
	public void systemStop(){
		byte[] out = {(byte) 0x00, (byte) 0x00, (byte) 0x07, (byte) 0x78, (byte) 0x05, (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x00};
		this.writeDatagram(ip, port, out);
	}
	
	// Set engine-speed of the given engine to the given value
	public void changeEngineSpeed(int mfxAddress, int newEngineSpeed){
		byte[] engineSpeed = {(byte) Math.floor(newEngineSpeed/256), (byte) Math.floor(newEngineSpeed%256)};
		byte[] address = {(byte) Math.floor(mfxAddress/256), (byte) Math.floor(mfxAddress%256)};
		byte[] out = {(byte) 0x00, (byte) 0x08, (byte) 0x07, (byte) 0x78, (byte) 0x06, (byte) 0x00, (byte) 0x00, address[0], address[1], engineSpeed[0], engineSpeed[1], (byte) 0x00, (byte) 0x00};
		this.writeDatagram(ip, port, out);
	}
	
	// Set engine-direction of the given engine to the given value
	public void changeEngineDirection(int mfxAddress, int newEngineDirection){
		byte engineDirection = (byte) Math.floor(newEngineDirection%256);
		byte[] address = {(byte) Math.floor(mfxAddress/256), (byte) Math.floor(mfxAddress%256)};
		byte[] out = {(byte) 0x00, (byte) 0x0A, (byte) 0x07, (byte) 0x78, (byte) 0x05, (byte) 0x00, (byte) 0x00, address[0], address[1], engineDirection, (byte) 0x00, (byte) 0x00, (byte) 0x00};
		this.writeDatagram(ip, port, out);
	}
	
	// Set the switch-direction of the given switch to the given value or move the turntable clockwise or counterclockwise
	public void changeSwitchDirection(int mfxAddress, int newSwitchDirection){
		byte switchDirection = (byte) Math.floor(newSwitchDirection%256);
		byte[] address = {(byte) Math.floor(mfxAddress/256), (byte) Math.floor(mfxAddress%256)};
		byte[] out = {(byte) 0x00, (byte) 0x16, (byte) 0x07, (byte) 0x78, (byte) 0x06, (byte) 0x00, (byte) 0x00, address[0], address[1], switchDirection, (byte) 0x01, (byte) 0x00, (byte) 0x00};
		this.writeDatagram(ip, port, out);
	}
	
}
