package maerklinProject_GUI;

import java.io.*;
import java.net.Socket;

import org.controlsfx.control.ToggleSwitch;

import javafx.application.Platform;
import javafx.beans.value.*;

import javafx.collections.FXCollections;
import javafx.fxml.*;
import javafx.scene.control.*;

import maerklinProject_Properties.ProjectProperties;

public class maerklinProject_GUIController {
	
	private Socket client;
	private InputStream in;
	private OutputStream out;
	
	private int selectedEngine;
	
	private boolean connectionEstablished;
	
	@FXML
	TextField serverIP;
	
	@FXML
	Button connectButton;
	
	@FXML
	ChoiceBox<String> engineChoiceBox;
	
	@FXML
	Slider engineSpeed;
	
	@FXML
	ToggleSwitch swToggle04;
	ToggleSwitch swToggle05;
	ToggleSwitch swToggle06;
	ToggleSwitch swToggle08;
	ToggleSwitch swToggle09;
	ToggleSwitch swToggle10;
	ToggleSwitch swToggle12;
	ToggleSwitch swToggle13;
	ToggleSwitch swToggle14;
	ToggleSwitch swToggle15;
	
	@FXML
	protected void initialize () {
		connectionEstablished = false;
		selectedEngine = ProjectProperties.ICE_ID;
		
		swToggle04.selectedProperty().addListener(new switchListener(this, swToggle04));
		swToggle05.selectedProperty().addListener(new switchListener(this, swToggle05));
		swToggle06.selectedProperty().addListener(new switchListener(this, swToggle06));
		swToggle08.selectedProperty().addListener(new switchListener(this, swToggle08));
		swToggle09.selectedProperty().addListener(new switchListener(this, swToggle09));
		swToggle10.selectedProperty().addListener(new switchListener(this, swToggle10));
		swToggle12.selectedProperty().addListener(new switchListener(this, swToggle12));
		swToggle13.selectedProperty().addListener(new switchListener(this, swToggle13));
		swToggle14.selectedProperty().addListener(new switchListener(this, swToggle14));
		swToggle15.selectedProperty().addListener(new switchListener(this, swToggle15));
		
		engineChoiceBox.setItems(FXCollections.observableArrayList("ICE", "Lok", "Reichsbahn"));
		engineChoiceBox.setValue("ICE");
		engineChoiceBox.getSelectionModel().selectedIndexProperty().addListener(new ChangeListener<Number>() {

			@Override
			public void changed (ObservableValue<? extends Number> ov, Number value, Number new_value) {
				switch (new_value.intValue()) {
					case 0:
						selectedEngine = ProjectProperties.ICE_ID;
						break;
					case 1:
						selectedEngine = ProjectProperties.LOK_ID;
						break;
					case 2:
						selectedEngine = ProjectProperties.REICHSBAHN_ID;
						break;
				}
			}
			
		});
		
        engineSpeed.valueProperty().addListener(new ChangeListener<Number>() {
        	
        	@Override
            public void changed (ObservableValue<? extends Number> ov,
                Number old_val, Number new_val) {
                    engineSetSpeed(new_val.intValue()*10);
            }
        });
     
	}
	
	@FXML
	protected void establishConnetion () {
		if (connectionEstablished) {
			try {
				byte[] datagram = {(byte) 0x00, (byte) ProjectProperties.SESSION_ABORT, (byte) 0x00, (byte) 0x00};
				out.write(datagram);
				client.close();
				connectionEstablished = false;
				serverIP.setStyle("-fx-background-color: red");
				connectButton.setText("Verbindung herstellen");
				
			} catch (Exception e) {
				System.out.println("Error at: establishConnection (shutting down connection");
				e.printStackTrace();
			}
		}
		else {
			try (Socket socket = new Socket(serverIP.getText(), ProjectProperties.PORT)) {
				client = socket;
				client.setKeepAlive(true);
				in = client.getInputStream();
				out = client.getOutputStream();
				client.setSoTimeout(200);
				connectionEstablished = true;
				serverIP.setStyle("-fx-background-color: green");
				connectButton.setText("Verbindung trennen");
				System.out.println("Verbindung zu "+client.getRemoteSocketAddress()+" aufgebaut!");
				Platform.runLater(new updateThread(this));
			}
			catch (Exception e) {
				System.out.println("Error at: establishConnection (establishing connection");
				e.printStackTrace();
			}
		}
	}

	@FXML
	protected void emergencyStop () {
		if (connectionEstablished) {
			try {
				byte[] datagram = {(byte) 0x00, (byte) ProjectProperties.SYSTEM_STOP, (byte) 0x00, (byte) 0x00};
				out.write(datagram);
			} catch (Exception e) {
				System.out.println("Error at: emergencyStop");
				e.printStackTrace();
			}
		}
	}
	
	@FXML
	protected void turntableTurnClockwise () {
		if (connectionEstablished) {
			try {
				byte[] datagram = {(byte) ProjectProperties.TURNTABLE_ID, (byte) ProjectProperties.SWITCH_SET_DIRECTION, (byte) 0x00, (byte) 0x00};
				out.write(datagram);
			} catch (Exception e) {
				System.out.println("Error at: emergencyStop");
				e.printStackTrace();
			}
		}
	}
	
	@FXML
	protected void turntableTurnCounterClockwise () {
		if (connectionEstablished) {
			try {
				byte[] datagram = {(byte) ProjectProperties.TURNTABLE_ID, (byte) ProjectProperties.SWITCH_SET_DIRECTION, (byte) 0x01, (byte) 0x00};
				out.write(datagram);
			} catch (Exception e) {
				System.out.println("Error at: emergencyStop");
				e.printStackTrace();
			}
		}
	}
	
	@FXML
	protected void engineSetDirectionRight () {
		if (connectionEstablished) {
			try {
				byte[] datagram = {(byte) selectedEngine, (byte) ProjectProperties.ENGINE_SET_DIRECTION, (byte) 0x01, (byte) 0x00};
				out.write(datagram);
			} catch (Exception e) {
				System.out.println("Error at: emergencyStop");
				e.printStackTrace();
			}
		}
	}
	
	@FXML
	protected void engineSetDirectionLeft () {
		if (connectionEstablished) {
			try {
				byte[] datagram = {(byte) selectedEngine, (byte) ProjectProperties.ENGINE_SET_DIRECTION, (byte) 0x02, (byte) 0x00};
				out.write(datagram);
			} catch (Exception e) {
				System.out.println("Error at: emergencyStop");
				e.printStackTrace();
			}
		}
	}
	
	protected void engineSetSpeed (int newEngineSpeed) {
		if (connectionEstablished) {
			try {
				byte[] datagram = {(byte) selectedEngine, (byte) ProjectProperties.ENGINE_SET_SPEED, (byte) ((newEngineSpeed/(1<<8))%(1<<8)), (byte) (newEngineSpeed%(1<<8))};
				out.write(datagram);
			} catch (Exception e) {
				System.out.println("Error at: emergencyStop");
				e.printStackTrace();
			}
		}
	}
	
	protected void switchSetDirection (int switchID, int newSwitchDirection) {
		if (connectionEstablished) {
			try {
				byte[] datagram = {(byte) switchID, (byte) ProjectProperties.SWITCH_SET_DIRECTION, (byte) newSwitchDirection, (byte) 0x00};
				out.write(datagram);
			} catch (Exception e) {
				System.out.println("Error at: emergencyStop");
				e.printStackTrace();
			}
		}
	}
	
	/*@FXML
	protected void switch04SetDirectionRight () {
		if (connectionEstablished) {
			try {
				byte[] datagram = {(byte) ProjectProperties.SWITCH4_ID, (byte) ProjectProperties.SWITCH_SET_DIRECTION, (byte) 0x00, (byte) 0x00};
				out.write(datagram);
			} catch (Exception e) {
				System.out.println("Error at: emergencyStop");
				e.printStackTrace();
			}
		}
	}
	
	@FXML
	protected void switch04SetDirectionLeft () {
		if (connectionEstablished) {
			try {
				byte[] datagram = {(byte) ProjectProperties.SWITCH4_ID, (byte) ProjectProperties.SWITCH_SET_DIRECTION, (byte) 0x01, (byte) 0x00};
				out.write(datagram);
			} catch (Exception e) {
				System.out.println("Error at: emergencyStop");
				e.printStackTrace();
			}
		}
	}
	
	@FXML
	protected void switch05SetDirectionRight () {
		if (connectionEstablished) {
			try {
				byte[] datagram = {(byte) ProjectProperties.SWITCH5_ID, (byte) ProjectProperties.SWITCH_SET_DIRECTION, (byte) 0x00, (byte) 0x00};
				out.write(datagram);
			} catch (Exception e) {
				System.out.println("Error at: emergencyStop");
				e.printStackTrace();
			}
		}
	}
	
	@FXML
	protected void switch05SetDirectionLeft () {
		if (connectionEstablished) {
			try {
				byte[] datagram = {(byte) ProjectProperties.SWITCH5_ID, (byte) ProjectProperties.SWITCH_SET_DIRECTION, (byte) 0x01, (byte) 0x00};
				out.write(datagram);
			} catch (Exception e) {
				System.out.println("Error at: emergencyStop");
				e.printStackTrace();
			}
		}
	}
	
	@FXML
	protected void switch06SetDirectionRight () {
		if (connectionEstablished) {
			try {
				byte[] datagram = {(byte) ProjectProperties.SWITCH6_ID, (byte) ProjectProperties.SWITCH_SET_DIRECTION, (byte) 0x00, (byte) 0x00};
				out.write(datagram);
			} catch (Exception e) {
				System.out.println("Error at: emergencyStop");
				e.printStackTrace();
			}
		}
	}
	
	@FXML
	protected void switch06SetDirectionLeft () {
		if (connectionEstablished) {
			try {
				byte[] datagram = {(byte) ProjectProperties.SWITCH6_ID, (byte) ProjectProperties.SWITCH_SET_DIRECTION, (byte) 0x01, (byte) 0x00};
				out.write(datagram);
			} catch (Exception e) {
				System.out.println("Error at: emergencyStop");
				e.printStackTrace();
			}
		}
	}
	
	@FXML
	protected void switch08SetDirectionRight () {
		if (connectionEstablished) {
			try {
				byte[] datagram = {(byte) ProjectProperties.SWITCH8_ID, (byte) ProjectProperties.SWITCH_SET_DIRECTION, (byte) 0x00, (byte) 0x00};
				out.write(datagram);
			} catch (Exception e) {
				System.out.println("Error at: emergencyStop");
				e.printStackTrace();
			}
		}
	}
	
	@FXML
	protected void switch08SetDirectionLeft () {
		if (connectionEstablished) {
			try {
				byte[] datagram = {(byte) ProjectProperties.SWITCH8_ID, (byte) ProjectProperties.SWITCH_SET_DIRECTION, (byte) 0x01, (byte) 0x00};
				out.write(datagram);
			} catch (Exception e) {
				System.out.println("Error at: emergencyStop");
				e.printStackTrace();
			}
		}
	}
	
	@FXML
	protected void switch09SetDirectionRight () {
		if (connectionEstablished) {
			try {
				byte[] datagram = {(byte) ProjectProperties.SWITCH9_ID, (byte) ProjectProperties.SWITCH_SET_DIRECTION, (byte) 0x00, (byte) 0x00};
				out.write(datagram);
			} catch (Exception e) {
				System.out.println("Error at: emergencyStop");
				e.printStackTrace();
			}
		}
	}
	
	@FXML
	protected void switch09SetDirectionLeft () {
		if (connectionEstablished) {
			try {
				byte[] datagram = {(byte) ProjectProperties.SWITCH9_ID, (byte) ProjectProperties.SWITCH_SET_DIRECTION, (byte) 0x01, (byte) 0x00};
				out.write(datagram);
			} catch (Exception e) {
				System.out.println("Error at: emergencyStop");
				e.printStackTrace();
			}
		}
	}
	
	@FXML
	protected void switch10SetDirectionRight () {
		if (connectionEstablished) {
			try {
				byte[] datagram = {(byte) ProjectProperties.SWITCH10_ID, (byte) ProjectProperties.SWITCH_SET_DIRECTION, (byte) 0x00, (byte) 0x00};
				out.write(datagram);
			} catch (Exception e) {
				System.out.println("Error at: emergencyStop");
				e.printStackTrace();
			}
		}
	}
	
	@FXML
	protected void switch10SetDirectionLeft () {
		if (connectionEstablished) {
			try {
				byte[] datagram = {(byte) ProjectProperties.SWITCH10_ID, (byte) ProjectProperties.SWITCH_SET_DIRECTION, (byte) 0x01, (byte) 0x00};
				out.write(datagram);
			} catch (Exception e) {
				System.out.println("Error at: emergencyStop");
				e.printStackTrace();
			}
		}
	}
	
	@FXML
	protected void switch12SetDirectionRight () {
		if (connectionEstablished) {
			try {
				byte[] datagram = {(byte) ProjectProperties.SWITCH12_ID, (byte) ProjectProperties.SWITCH_SET_DIRECTION, (byte) 0x00, (byte) 0x00};
				out.write(datagram);
			} catch (Exception e) {
				System.out.println("Error at: emergencyStop");
				e.printStackTrace();
			}
		}
	}
	
	@FXML
	protected void switch12SetDirectionLeft () {
		if (connectionEstablished) {
			try {
				byte[] datagram = {(byte) ProjectProperties.SWITCH12_ID, (byte) ProjectProperties.SWITCH_SET_DIRECTION, (byte) 0x01, (byte) 0x00};
				out.write(datagram);
			} catch (Exception e) {
				System.out.println("Error at: emergencyStop");
				e.printStackTrace();
			}
		}
	}
	
	@FXML
	protected void switch13SetDirectionRight () {
		if (connectionEstablished) {
			try {
				byte[] datagram = {(byte) ProjectProperties.SWITCH13_ID, (byte) ProjectProperties.SWITCH_SET_DIRECTION, (byte) 0x00, (byte) 0x00};
				out.write(datagram);
			} catch (Exception e) {
				System.out.println("Error at: emergencyStop");
				e.printStackTrace();
			}
		}
	}
	
	@FXML
	protected void switch13SetDirectionLeft () {
		if (connectionEstablished) {
			try {
				byte[] datagram = {(byte) ProjectProperties.SWITCH13_ID, (byte) ProjectProperties.SWITCH_SET_DIRECTION, (byte) 0x01, (byte) 0x00};
				out.write(datagram);
			} catch (Exception e) {
				System.out.println("Error at: emergencyStop");
				e.printStackTrace();
			}
		}
	}
	
	@FXML
	protected void switch14SetDirectionRight () {
		if (connectionEstablished) {
			try {
				byte[] datagram = {(byte) ProjectProperties.SWITCH14_ID, (byte) ProjectProperties.SWITCH_SET_DIRECTION, (byte) 0x00, (byte) 0x00};
				out.write(datagram);
			} catch (Exception e) {
				System.out.println("Error at: emergencyStop");
				e.printStackTrace();
			}
		}
	}
	
	@FXML
	protected void switch14SetDirectionLeft () {
		if (connectionEstablished) {
			try {
				byte[] datagram = {(byte) ProjectProperties.SWITCH14_ID, (byte) ProjectProperties.SWITCH_SET_DIRECTION, (byte) 0x01, (byte) 0x00};
				out.write(datagram);
			} catch (Exception e) {
				System.out.println("Error at: emergencyStop");
				e.printStackTrace();
			}
		}
	}
	
	@FXML
	protected void switch15SetDirectionRight () {
		if (connectionEstablished) {
			try {
				byte[] datagram = {(byte) ProjectProperties.SWITCH15_ID, (byte) ProjectProperties.SWITCH_SET_DIRECTION, (byte) 0x00, (byte) 0x00};
				out.write(datagram);
			} catch (Exception e) {
				System.out.println("Error at: emergencyStop");
				e.printStackTrace();
			}
		}
	}
	
	@FXML
	protected void switch15SetDirectionLeft () {
		if (connectionEstablished) {
			try {
				byte[] datagram = {(byte) ProjectProperties.SWITCH15_ID, (byte) ProjectProperties.SWITCH_SET_DIRECTION, (byte) 0x01, (byte) 0x00};
				out.write(datagram);
			} catch (Exception e) {
				System.out.println("Error at: emergencyStop");
				e.printStackTrace();
			}
		}
	}*/
	
}

class switchListener implements ChangeListener<Boolean> {
	
	private maerklinProject_GUIController controllerInstance;
	private ToggleSwitch node;
	private int switchID;
	private int newSwitchDirection;
	
	switchListener (maerklinProject_GUIController controller, ToggleSwitch element) {
		controllerInstance = controller;
		node = element;
	}

	@Override
	public void changed (ObservableValue<? extends Boolean> observable, Boolean oldValue, Boolean newValue) {
		switchID = Integer.parseInt(""+node.getId().charAt(8)+node.getId().charAt(9));
		if (newValue) {
			newSwitchDirection = 1;
		}
		else {
			newSwitchDirection = 0;
		}
		controllerInstance.switchSetDirection(switchID, newSwitchDirection);
	}
	
}

class updateThread implements Runnable {
	
	maerklinProject_GUIController controllerInstance;
	
	updateThread (maerklinProject_GUIController controller) {
		controllerInstance = controller;
	}

	@Override
	public void run() {
		// TODO Auto-generated method stub
		
	}
	
}
