package maerklinProject_ServerApplication;


import java.net.DatagramPacket;
import java.net.DatagramSocket;
import javax.xml.bind.DatatypeConverter;

public class UDPListener implements Runnable{
	
	private final static int port = 15730;
	
	private byte[] in = new byte[13];
	
	
	public UDPListener(){
		//Nothing to do...
	}
	
	public void listen(){
		try(DatagramSocket socket = new DatagramSocket(port)){
			socket.setReuseAddress(true);
			DatagramPacket incomingData = new DatagramPacket(in, in.length);
			socket.receive(incomingData);
			in = incomingData.getData();
		} 
		catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	public byte[] getData(){
		return in;
	}
	
	public int getEngineSpeed(){
		return (((in[9]&0x7f)<<8)+(in[9]>>7)*65536+(in[10]&0x7f)+(in[10]>>7)*256);
	}
	
	public int getEngineDirection(){
		return ((in[9]&0x7f)+(in[9]>>7)*256);
	}
	
	public int getSwitchDirection(){
		return ((in[9]&0x7f)+(in[9]>>7)*256);
	}
	
	public int getAddress(){
		return (((in[7]&0x7f)<<8)+(in[7]>>7)*65536+(in[8]&0x7f)+(in[8]>>7)*256);
	}
	
	public int getCommand(){
		return ((in[1]&0x7e)+(in[1]>>7)*256);
	}
	
	public boolean isAnswer(){
		if ((in[1]&0x01) == 1){
			return true;
		}
		else {
			return false;
		}
	}
	
	public void printData(byte[] data){
		try{
			byte[] kennung = new byte[4];
			byte[] laenge = new byte[1];
			byte[] nutzdaten = new byte[8];
			System.arraycopy(data, 0, kennung, 0, 4);
			System.arraycopy(data, 4, laenge, 0, 1);
			System.arraycopy(data, 5, nutzdaten, 0, 8);
			System.out.print("\n\nKennung   DLC Nutzdaten\n" + DatatypeConverter.printHexBinary(kennung) + "  " + DatatypeConverter.printHexBinary(laenge) + "  " + DatatypeConverter.printHexBinary(nutzdaten));
		}
		catch(Exception e){
			e.printStackTrace();
		}
	}
	
	public static void main(String args[]){
		UDPListener testListener = new UDPListener();
		while (true){
			testListener.listen();
			testListener.printData(testListener.getData());
		}
	}

	@Override
	public void run() {
		while (true){
			listen();
			if (isAnswer()){
				if (getCommand() == 0x08){
					for (Engine train : Engine.engines){
						if (train.getEngineID() == getAddress()){
							train.engineSpeed = getEngineSpeed();
							System.out.println("Speed of Engine "+train.getEngineID()+" changed to "+train.getEngineSpeed());
							break;
						}
					}
				}
				else if (getCommand() == 0x0A){
					for (Engine train : Engine.engines){
						if (train.getEngineID() == getAddress()){
							train.engineDirection = getEngineDirection();
							System.out.println("Direction of Engine "+train.getEngineID()+" changed to "+train.getEngineDirection());
							break;
						}
					}
				}
				else if (getCommand() == 0x16){
					for (Switch sw : Switch.switches){
						if (sw.getSwitchID() == getAddress()){
							sw.switchDirection = getEngineDirection();
							System.out.println("Direction of Switch "+sw.getSwitchID()+" changed to "+sw.getSwitchDirection());
							break;
						}
					}
				}
			}
		}
	}
	
}
