package tictactoePraktikum;

import org.junit.*;
import static org.junit.Assert.*;

/**
 * @author Lukas Friedrichsen
 *
 */
public class TicTacToeTest {
	
	private static TicTacToe ttt;
	private static MockUpClass mockUp;
	
	/**
	 * @throws java.lang.Exception
	 */
	@BeforeClass
	public static void SetUpBeforeClass() throws Exception {
		mockUp = new MockUpClass();
		ttt = new TicTacToe(mockUp);
	}

	/**
	 * @throws java.lang.Exception
	 */
	@AfterClass
	public static void tearDownAfterClass() throws Exception {
		ttt = null;
	}

	/**
	 * @throws java.lang.Exception
	 */
	@Before
	public void setUp() throws Exception {
		ttt.resetSpiel();
	}

	/**
	 * Test method for {@link tictactoePraktikum.TicTacToe#getNaechstenSpieler(int)}.
	 */
	@Test
	public void testGetNaechstenSpieler() {
		assertEquals(1, ttt.getNaechstenSpieler(0));
		assertEquals(0, ttt.getNaechstenSpieler(1));
		assertEquals(1, ttt.getNaechstenSpieler(42));
	}

	/**
	 * Test method for {@link tictactoePraktikum.TicTacToe#isGueltig(int, int)}.
	 */
	@Test
	public void testIsGueltig() {
		char[][] spielbrett = {{' ','X','O'},{'O',' ',' '},{'X',' ',' '}};
		ttt.setSpielbrett(spielbrett);
		assertFalse(ttt.isGueltig(-1, 0));
		assertFalse(ttt.isGueltig(0, -1));
		assertTrue(ttt.isGueltig(0, 0));
		assertFalse(ttt.isGueltig(0, 1));
		assertFalse(ttt.isGueltig(0, 2));
		assertFalse(ttt.isGueltig(0, 3));
		assertFalse(ttt.isGueltig(3, 0));
	}

	/**
	 * Test method for {@link tictactoePraktikum.TicTacToe#getSpielzustand()}.
	 */
	@Test
	public void testGetSpielzustand() {
		char[][] spielbrett0 = {{'X','X','X'},{' ',' ',' '},{' ',' ',' '}};
		ttt.setSpielbrett(spielbrett0);
		assertEquals(1, ttt.getSpielzustand());
		char[][] spielbrett1 = {{'X',' ',' '},{'X',' ',' '},{'X',' ',' '}};
		ttt.setSpielbrett(spielbrett1);
		assertEquals(1, ttt.getSpielzustand());
		char[][] spielbrett2 = {{'X',' ',' '},{' ','X',' '},{' ',' ','X'}};
		ttt.setSpielbrett(spielbrett2);
		assertEquals(1, ttt.getSpielzustand());
		char[][] spielbrett3 = {{' ',' ','X'},{' ','X',' '},{'X',' ',' '}};
		ttt.setSpielbrett(spielbrett3);
		assertEquals(1, ttt.getSpielzustand());
		char[][] spielbrett4 = {{'O','O','O'},{' ',' ',' '},{' ',' ',' '}};
		ttt.setSpielbrett(spielbrett4);
		assertEquals(1, ttt.getSpielzustand());
		char[][] spielbrett5 = {{'O',' ',' '},{'O',' ',' '},{'O',' ',' '}};
		ttt.setSpielbrett(spielbrett5);
		assertEquals(1, ttt.getSpielzustand());
		char[][] spielbrett6 = {{'O',' ',' '},{' ','O',' '},{' ',' ','O'}};
		ttt.setSpielbrett(spielbrett6);
		assertEquals(1, ttt.getSpielzustand());
		char[][] spielbrett7 = {{' ',' ','O'},{' ','O',' '},{'O',' ',' '}};
		ttt.setSpielbrett(spielbrett7);
		assertEquals(1, ttt.getSpielzustand());
		char[][] spielbrett8 = {{'X','X','O'},{'O','O','X'},{'X','X','O'}};
		ttt.setSpielbrett(spielbrett8);
		assertEquals(2, ttt.getSpielzustand());
		char[][] spielbrett9 = {{' ',' ',' '},{' ',' ',' '},{' ',' ',' '}};
		ttt.setSpielbrett(spielbrett9);
		assertEquals(3, ttt.getSpielzustand());
	}

	/**
	 * Test method for {@link tictactoePraktikum.TicTacToe#spielzug(int)}.
	 */
	@Test
	public void testSpielzug() {
		char[][] spielbrett1 = {{' ',' ',' '},{' ','X',' '},{' ',' ',' '}};
		char[][] spielbrett2 = {{'O',' ',' '},{' ','X',' '},{' ',' ',' '}};
		char[][] spielbrett3 = {{'O',' ',' '},{' ','X',' '},{' ',' ','O'}};
		mockUp.testcase = 1;
		mockUp.index = -1;
		ttt.setSpielbrett(spielbrett1);
		assertEquals(4, ttt.spielzug(1));
		assertArrayEquals(spielbrett1, ttt.getSpielbrett());
		assertEquals(3, ttt.spielzug(1));
		assertArrayEquals(spielbrett2, ttt.getSpielbrett());
		assertEquals(3, ttt.spielzug(1));
		assertEquals(3, mockUp.index);
		assertArrayEquals(spielbrett3, ttt.getSpielbrett());
	}
	
	/**
	 * Test method for {@link tictactoePraktikum.TicTacToe#spielzug(int)} for an IndexOutOfBoundsException.
	 */
	@Test (expected=IndexOutOfBoundsException.class)
	public void testSpielzugForIndexOutOFBoundsException() {
		ttt.spielzug(3);
	}

	/**
	 * Test method for {@link tictactoePraktikum.TicTacToe#getGewinner()}.
	 */
	@Test
	public void testGetGewinner() {
		assertEquals(' ', ttt.getGewinner());
		char[][] spielbrettX = {{'X','X','X'},{' ',' ',' '},{' ',' ',' '}};
		ttt.setSpielbrett(spielbrettX);
		assertEquals('X', ttt.getGewinner());
		char[][] spielbrettO = {{'O','O','O'},{' ',' ',' '},{' ',' ',' '}};
		ttt.setSpielbrett(spielbrettO);
		assertEquals('O', ttt.getGewinner());
	}
	
	/**
	 * Test method for {@link tictactoePraktikum.TicTacToe#spiel()}.
	 */
	@Test
	public void testSpiel() {
		char[][] spielbrett = {{' ',' ',' '},{' ',' ',' '},{' ',' ',' '}};
		ttt.setSpielbrett(spielbrett);
		mockUp.testcase = 2;
		mockUp.index = -1;
		ttt.spiel(0);
		assertEquals('X', ttt.getGewinner());
		assertEquals(7, ttt.getAnzahlZuege());
	}
}
