package gui.model;

import javafx.beans.property.IntegerProperty;
import gui.view.SwitchControl;
import javafx.beans.property.BooleanProperty;
import javafx.beans.property.SimpleIntegerProperty;
import javafx.beans.property.SimpleObjectProperty;
import javafx.beans.property.SimpleBooleanProperty;
import javafx.beans.property.SimpleStringProperty;
import javafx.beans.property.StringProperty;
import javafx.beans.property.ObjectProperty;

/**
 * Model class for a Model Train Switch.
 *
 * @author Philipp Stenkamp
 */
public class Switch{

    private final StringProperty name;
    private final IntegerProperty id;
    private final BooleanProperty state;
    private final ObjectProperty<SwitchControl> controller;

    /**
     * Default constructor.
     */
    public Switch() {
        this(null, null, null);
    }

    /**
     * Constructor with some initial data.
     * 
     * @param name
     * @param id
     */
    
    public Switch(String name, Integer id){
    	this(name, id, null);
    }
    
    public Switch(String name, Integer id, SwitchControl controller) {
        this.name = new SimpleStringProperty(name);
        this.id = new SimpleIntegerProperty(id);
        this.controller = new SimpleObjectProperty<SwitchControl>(controller);
        
        // Initialize the switch in a straigt (=false) state
        this.state = new SimpleBooleanProperty(false);
      
	}
    
    @Override
    public String toString() {
      return name.getValue();
    }
    
    public StringProperty getName() {
		return name;
	}
    
    public void setName(String name) {
		this.name.set(name);
	}

	public IntegerProperty getMaerklinID() {
		return id;
	}

	public BooleanProperty getState() {
		return state;
	}
	
	public void setState(Boolean state) {
		this.state.set(state);
	}
	
	public ObjectProperty<SwitchControl> getController(){
		return controller;
	}
	
	public void setController(SwitchControl controller){
		this.controller.setValue(controller);
	}
}
