package server;


import java.util.ArrayList;

public class Engine {
	
	static ArrayList<Engine> engines;
	
	private int engineID;
	int engineDirection;
	int engineSpeed;
	
	private MaerklinProtocol udpProtocol;
	private UDPListener listener;
	
	public Engine(){
		engines = new ArrayList<Engine>();
		udpProtocol = new MaerklinProtocol();
		listener = new UDPListener();
		engines.add(this);
	}
	
	public Engine(int id, int direction, int speed){
		engines = new ArrayList<Engine>();
		udpProtocol = new MaerklinProtocol();
		listener = new UDPListener();
		engines.add(this);
		try {
			setEngineID(id);
			setEngineDirection(direction);
			setEngineSpeed(speed);
		}
		catch(Exception e){
			System.out.println(e.getMessage());
		}
	}
	
	public void setEngineID(int newID){
		engineID = newID;
	}
	
	public void setEngineDirection(int newEngineDirection) throws Exception {
		if (newEngineDirection == 1 || newEngineDirection == 2){
			engineDirection = newEngineDirection;
		}
		else if (newEngineDirection == 3){
			if (engineDirection == 1){
				engineDirection = 2;
			}
			else {
				engineDirection = 1;
			}
		}
		else {
			Exception e = new Exception("No viable direction!");
			throw e;
		}
		udpProtocol.changeEngineDirection(engineID, engineDirection);
		listener.listen();
		if (engineDirection != listener.getEngineDirection()){
			Exception e = new Exception("Communication Error!");
			throw e;
		}
	}
	
	public void setEngineSpeed(int newEngineSpeed) throws Exception {
		if (newEngineSpeed <= 1000 && newEngineSpeed >= 0){
			engineSpeed = newEngineSpeed;
		}
		else {
			Exception e = new Exception("No viable speed value!");
			throw e;
		}
		udpProtocol.changeEngineSpeed(engineID, engineSpeed);
		listener.listen();
		if (engineSpeed != listener.getEngineSpeed()){
			Exception e = new Exception("Communication Error!");
			throw e;
		}
	}
	
	public void increaseEngineSpeed() throws Exception {
		engineSpeed = engineSpeed+50;
		udpProtocol.changeEngineSpeed(engineID, engineSpeed);
		listener.listen();
		if (engineSpeed != listener.getEngineSpeed()){
			Exception e = new Exception("Communication Error!");
			throw e;
		}
	}
	
	public void decreaseEngineSpeed() throws Exception{
		engineSpeed = engineSpeed-50;
		udpProtocol.changeEngineSpeed(engineID, engineSpeed);
		listener.listen();
		if (engineSpeed != listener.getEngineSpeed()){
			Exception e = new Exception("Communication Error!");
			throw e;
		}
	}
	
	public int getEngineID(){
		return engineID;
	}
	
	public int getEngineDirection(){
		return engineDirection;
	}
	
	public int getEngineSpeed(){
		return engineSpeed;
	}

}
